var types = function(type) {
  return function(value) {
    return Object.prototype.toString.call(value) === `[object ${type}]`
  }
}

var isNull = types('Null')
var isUndefined = types('Undefined')

var isString = types('String')
var isBoolean = types('Boolean')

var isPlainObject = types('Object')
var isArray = Array.isArray || types('Array')

var isObject = function(target) {
  return !isNull(target) && (isArray(target) || isPlainObject(target))
}

var isEqualNaN = function(target) {
  return target !== target
}

var isNumber = function(num) {
  var number = +num

  // Infinity, NaN
  if ((number - number) !== 0) {
    return false
  }

  // Number
  if (number === num) {
    return true
  }

  if (isString(num)) {
    if (num.trim() === '') {
      return false
    }

    return true
  }

  return false
}

var isEmpty = function(target) {
  if (isString(target)) {
    return target.trim() === ''
  }

  // null, undefined, NaN
  if (isNull(target) || isUndefined(target) || isEqualNaN(target)) {
    return true
  }

  // array, object
  if (isObject(target)) {
    return Object.keys(target).length === 0
  }

  // boolean, number
  if (isBoolean(target) || isNumber(target)) {
    return false
  }

  return !target
}

var isFunction = function(target) {
  return typeof target === 'function' || types('Function')(target)
}

var isPromise = function(target) {
  const isXPromise = !!target && isFunction(target.then) && isFunction(target.catch)

  return isXPromise || types('Promise')(target)
}

module.exports = {
  isEmpty: isEmpty,

  isNull: isNull,
  isUndefined: isUndefined,

  isNumber: isNumber,
  isString: isString,
  isBoolean: isBoolean,

  isArray: isArray,
  isObject: isObject,
  isPlainObject: isPlainObject,

  isFunction: isFunction,
  isPromise: isPromise,

  isNaN: isEqualNaN
}
