var assert = require('assert')
var types = require('./')

describe('Test types:', () => {
  describe('#isEmpty()', () => {
    it('Test value `[]` is true.', () => assert.equal(types.isEmpty([]), true))
    it('Test value `{}` is true.', () => assert.equal(types.isEmpty({}), true))
    it('Test value `\' \'` is true.', () => assert.equal(types.isEmpty(' '), true))
    it('Test value `null` is true.', () => assert.equal(types.isEmpty(null), true))
    it('Test value `undefined` is true.', () => assert.equal(types.isEmpty(undefined), true))
    it('Test value `0` is false.', () => assert.equal(types.isEmpty(0), false))
    it('Test value `false` is false.', () => assert.equal(types.isEmpty(false), false))
  })

  describe('#isNull()', () => {
    it('Test value `null` is true.', () => assert.equal(types.isNull(null), true))
    it('Test value `\' \'` is false.', () => assert.equal(types.isNull(' '), false))
    it('Test value `undefined` is false.', () => assert.equal(types.isNull(undefined), false))
  })

  describe('#isUndefined()', () => {
    it('Test value `undefined` is true.', () => assert.equal(types.isUndefined(undefined), true))
    it('Test value `\' \'` is false.', () => assert.equal(types.isUndefined(' '), false))
    it('Test value `null` is false.', () => assert.equal(types.isUndefined(null), false))
  })

  describe('#isNumber()', () => {
    it('Test value `1` is true.', () => assert.equal(types.isNumber(1), true))
    it('Test value `\'1\'` is true.', () => assert.equal(types.isNumber('1'), true))
    it('Test value `NaN` is false.', () => assert.equal(types.isNumber(NaN), false))
    it('Test value `Infinity` is false.', () => assert.equal(types.isNumber(Infinity), false))
  })

  describe('#isString()', () => {
    it('Test value `\'1\'` is true.', () => assert.equal(types.isString('1'), true))
    it('Test value `1` is false.', () => assert.equal(types.isString(1), false))
    it('Test value `false` is false.', () => assert.equal(types.isString(false), false))
    it('Test value `NaN` is false.', () => assert.equal(types.isString(NaN), false))
    it('Test value `null` is false.', () => assert.equal(types.isString(null), false))
    it('Test value `[]` is false.', () => assert.equal(types.isString([]), false))
    it('Test value `{}` is false.', () => assert.equal(types.isString({}), false))
  })

  describe('#isBoolean()', () => {
    it('Test value `false` is true.', () => assert.equal(types.isBoolean(false), true))
    it('Test value `1` is false.', () => assert.equal(types.isBoolean(1), false))
    it('Test value `null` is false.', () => assert.equal(types.isBoolean(null), false))
    it('Test value `[]` is false.', () => assert.equal(types.isBoolean([]), false))
  })

  describe('#isObject()', () => {
    it('Test value `[]` is true.', () => assert.equal(types.isObject([]), true))
    it('Test value `{}` is true.', () => assert.equal(types.isObject({}), true))
    it('Test value `false` is false.', () => assert.equal(types.isObject(false), false))
    it('Test value `1` is false.', () => assert.equal(types.isObject(1), false))
    it('Test value `null` is false.', () => assert.equal(types.isObject(null), false))
  })

  describe('#isArray()', () => {
    it('Test value `[]` is true.', () => assert.equal(types.isArray([]), true))
    it('Test value `{}` is false.', () => assert.equal(types.isArray({}), false))
    it('Test value `false` is false.', () => assert.equal(types.isArray(false), false))
    it('Test value `1` is false.', () => assert.equal(types.isArray(1), false))
    it('Test value `null` is false.', () => assert.equal(types.isArray(null), false))
  })

  describe('#isPlainObject()', () => {
    it('Test value `{}` is true.', () => assert.equal(types.isPlainObject({}), true))
    it('Test value `[]` is false.', () => assert.equal(types.isPlainObject([]), false))
    it('Test value `false` is false.', () => assert.equal(types.isPlainObject(false), false))
    it('Test value `1` is false.', () => assert.equal(types.isPlainObject(1), false))
    it('Test value `null` is false.', () => assert.equal(types.isPlainObject(null), false))
  })

  describe('#isNaN()', () => {
    it('Test value `NaN` is true.', () => assert.equal(types.isNaN(NaN), true))
    it('Test value `Infinity` is false.', () => assert.equal(types.isNaN(Infinity), false))
    it('Test value `1` is false.', () => assert.equal(types.isNaN(1), false))
    it('Test value `{}` is false.', () => assert.equal(types.isNaN({}), false))
    it('Test value `false` is false.', () => assert.equal(types.isNaN(false), false))
    it('Test value `null` is false.', () => assert.equal(types.isNaN(null), false))
  })

  var foo = function() {}
  var promise = new Promise(resolve => resolve())

  describe('#isFunction()', () => {
    it('Test value `function foo() {}` is true.', () => assert.equal(types.isFunction(foo), true))
    it('Test value `new Promise(resolve => resolve())` is false.', () => assert.equal(types.isFunction(promise), false))
    it('Test value `{}` is false.', () => assert.equal(types.isFunction({}), false))
  })

  describe('#isPromise()', () => {
    it('Test value `new Promise(resolve => resolve())` is true.', () => assert.equal(types.isPromise(promise), true))
    it('Test value `function foo() {}` is false.', () => assert.equal(types.isPromise(foo), false))
    it('Test value `{}` is false.', () => assert.equal(types.isPromise({}), false))
  })
})
